@@ -31,7 +31,7 @@ class DataFlow(IntEnum):
 
 def valididentifier(item: str) -> bool:
     """Test 'item' for valid Python identifier"""
-    return bool(re.match(r"^[a-zA-Z][\w]+$", item))
+    return bool(re.match(r"^[a-zA-Z][\w]*$", item))
 
 
 class LV_Control(ABC):
@@ -194,8 +194,11 @@ class Cluster(LV_Control, Sequence):
         if "_ctrls" in self.__dict__ and item in self._ctrls:
             self._ctrls[item].value = value
         elif item == "value":
-            for c, v in zip(self._ctrls, value):
-                self._ctrls[c].value = v
+            if isinstance(value, dict):
+                self.update(value)
+            else:
+                for c, v in zip(self._ctrls, value):
+                    self._ctrls[c].value = v
         else:
             super().__setattr__(item, value)
 
@@ -254,7 +257,8 @@ class Cluster(LV_Control, Sequence):
 
     def update(self, controls: dict):
         """Update from a dict"""
-        self._ctrls.update(controls)
+        for name in controls:
+            self._ctrls[name].value = controls[name]
 
     def __dir__(self):
         attrs = super().__dir__()
@@ -265,7 +269,75 @@ class Cluster(LV_Control, Sequence):
                 ctrls.append(ctrl)
             else:
                 ctrls.append(f"['{ctrl}']")
-        return attrs + ctrls
+        methods = ["as_dict", "count", "index", "update", "value"]
+        return attrs + ctrls + methods
+
+    @property
+    def value(self):
+        """Return cluster's values as list"""
+        values = []
+        for _, ctrl in self._ctrls.items():
+            values.append(ctrl.value)
+        return values
+
+
+def is_ragged(array, res):
+    """Is 'array' ragged?"""
+
+    def _len(array):
+        sz = -1
+        try:
+            sz = len(array)
+        except TypeError:
+            pass
+        return sz
+
+    if _len(array) <= 0:
+        return res
+    elem0_sz = _len(array[0])
+    for element in array:
+        if _len(element) != elem0_sz:
+            res = True
+            break
+    for element in array:
+        res = res or is_ragged(element, res)
+    return res
+
+
+class WaveformGraph(LV_Control):
+    """'Waveform Graph' can be either 1d or 2d Array or Cluster"""
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self.value = kwargs.pop("value", np.array([]))
+
+    def __setattr__(self, item, value):
+        if item == "value":
+            if isinstance(value, str) or not hasattr(value, "__iter__"):
+                raise TypeError(f"'{value}' not array like")
+            if not is_ragged(value, False):
+                value = np.array(value)
+            else:
+                value = list(value)
+                super().__setattr__("t0", value[0])
+                super().__setattr__("dt", value[1])
+                super().__setattr__("Y", np.array(value[2]))
+        else:
+            try:
+                index = ["t0", "dt", "Y"].index(item)
+            except ValueError:
+                pass
+            else:
+                lvalue = super().__getattribute__("value")
+                lvalue[index] = value
+                super().__setattr__("value", lvalue)
+        super().__setattr__(item, value)
+
+    def __repr__(self):
+        return f"{self.value}"
+
+    def __str__(self):
+        return f"{self.value}"
 
 
 class String(LV_Control):
@@ -394,13 +466,47 @@ class SharedVariable(IORefNum):
     """Shared Variable"""
 
 
+class UDRefNum(IORefNum):
+    """User Defined RefNum"""
+
+
+class Ring(LV_Control):
+    """Ring"""
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self.value = kwargs.pop("value", 0)
+        self.items = kwargs.pop("items", None)
+
+    def __setattr__(self, item, value):
+        if item == "value":
+            if isinstance(value, str):
+                value = self.items.index(value)
+        super().__setattr__(item, value)
+
+    def __repr__(self):
+        return f"{self.items[self.value]}"
+
+    def __str__(self):
+        return f"{self.items[self.value]}"
+
+
+class NotImplControl(LV_Control):
+    """Control Not Implemented"""
+
+    def __repr__(self):
+        return "Not Implemented"
+
+    def __str__(self):
+        return "Not Implemented"
+
+
 LVControl_LU = {
     "Numeric": Numeric,
     "Boolean": Boolean,
     "String": String,
     "Path": Path,
     "Time Stamp": TimeStamp,
-    "Waveform Graph": LV_Control,
     "Enum": Enum,
     "IVI Logical Name": IVILogicalName,
     "Slide": Numeric,
@@ -410,10 +516,15 @@ LVControl_LU = {
     "Classic DAQmx Physical Channel": String,
     "VISA resource name": VISAResourceName,
     "Classic Shared Variable Control": SharedVariable,
+    "Ring": Ring,
+    "User Defined Refnum Tag": UDRefNum,
+    "Waveform Graph": WaveformGraph,
+    "XY Graph": Array,
 }
 
 
 def make_control(**attrs: dict) -> LV_Control:
     """Make LV_Control from VI strings attributes"""
-    LV_Control_cls = LVControl_LU[attrs.pop("type")]
+    control_type = attrs.pop("type")
+    LV_Control_cls = LVControl_LU.get(control_type, NotImplControl)
     return LV_Control_cls(**attrs)
